@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 -- | Parsing of stack command line arguments
 
@@ -5,12 +6,13 @@ module Data.Attoparsec.Args
     ( EscapingMode(..)
     , argsParser
     , parseArgs
+    , parseArgsFromString
     ) where
 
-import           Control.Applicative
 import           Data.Attoparsec.Text ((<?>))
 import qualified Data.Attoparsec.Text as P
-import           Data.Text (Text)
+import qualified Data.Text as T
+import           Stack.Prelude
 
 -- | Mode for parsing escape characters.
 data EscapingMode
@@ -22,6 +24,10 @@ data EscapingMode
 parseArgs :: EscapingMode -> Text -> Either String [String]
 parseArgs mode = P.parseOnly (argsParser mode)
 
+-- | Parse using 'argsParser' from a string.
+parseArgsFromString :: EscapingMode -> String -> Either String [String]
+parseArgsFromString mode = P.parseOnly (argsParser mode) . T.pack
+
 -- | A basic argument parser. It supports space-separated text, and
 -- string quotation with identity escaping: \x -> x.
 argsParser :: EscapingMode -> P.Parser [String]
