@@ -2,136 +2,211 @@
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
-{-# LANGUAGE TupleSections         #-}
+{-# LANGUAGE RecordWildCards       #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE ConstraintKinds #-}
 -- Load information on package sources
 module Stack.Build.Source
-    ( loadSourceMap
-    , loadSourceMapFull
-    , SourceMap
+    ( projectLocalPackages
+    , localDependencies
+    , loadCommonPackage
+    , loadLocalPackage
+    , loadSourceMap
     , getLocalFlags
-    , getGhcOptions
     , addUnlistedToBuildCache
+    , hashSourceMapData
     ) where
 
 import              Stack.Prelude
-import              Crypto.Hash (Digest, SHA256(..))
-import              Crypto.Hash.Conduit (sinkHash)
-import qualified    Data.ByteArray as Mem (convert)
+import qualified    Pantry.SHA256 as SHA256
 import qualified    Data.ByteString as S
-import              Data.Conduit (ZipSink (..))
+import              Data.ByteString.Builder (toLazyByteString)
+import              Conduit (ZipSink (..), withSourceFile)
 import qualified    Data.Conduit.List as CL
-import qualified    Data.HashSet as HashSet
+import qualified    Distribution.PackageDescription as C
 import              Data.List
 import qualified    Data.Map as Map
 import qualified    Data.Map.Strict as M
 import qualified    Data.Set as Set
+import              Foreign.C.Types (CTime)
 import              Stack.Build.Cache
+import              Stack.Build.Haddock (shouldHaddockDeps)
 import              Stack.Build.Target
-import              Stack.Config (getLocalPackages)
-import              Stack.Constants (wiredInPackages)
 import              Stack.Package
-import              Stack.PackageLocation
+import              Stack.SourceMap
 import              Stack.Types.Build
-import              Stack.Types.BuildPlan
 import              Stack.Types.Config
-import              Stack.Types.FlagName
 import              Stack.Types.NamedComponent
 import              Stack.Types.Package
-import              Stack.Types.PackageName
-import qualified    System.Directory as D
+import              Stack.Types.SourceMap
 import              System.FilePath (takeFileName)
 import              System.IO.Error (isDoesNotExistError)
+import              System.PosixCompat.Files (modificationTime, getFileStatus)
 
--- | Like 'loadSourceMapFull', but doesn't return values that aren't as
--- commonly needed.
-loadSourceMap :: HasEnvConfig env
-              => NeedTargets
+-- | loads and returns project packages
+projectLocalPackages :: HasEnvConfig env
+              => RIO env [LocalPackage]
+projectLocalPackages = do
+    sm <- view $ envConfigL.to envConfigSourceMap
+    for (toList $ smProject sm) loadLocalPackage
+
+-- | loads all local dependencies - project packages and local extra-deps
+localDependencies :: HasEnvConfig env => RIO env [LocalPackage]
+localDependencies = do
+    bopts <- view $ configL.to configBuild
+    sourceMap <- view $ envConfigL . to envConfigSourceMap
+    forMaybeM (Map.elems $ smDeps sourceMap) $ \dp ->
+        case dpLocation dp of
+            PLMutable dir -> do
+                pp <- mkProjectPackage YesPrintWarnings dir (shouldHaddockDeps bopts)
+                Just <$> loadLocalPackage pp
+            _ -> return Nothing
+
+-- | Given the parsed targets and build command line options constructs
+--   a source map
+loadSourceMap :: HasBuildConfig env
+              => SMTargets
               -> BuildOptsCLI
-              -> RIO env ([LocalPackage], SourceMap)
-loadSourceMap needTargets boptsCli = do
-    (_, _, locals, _, sourceMap) <- loadSourceMapFull needTargets boptsCli
-    return (locals, sourceMap)
-
--- | Given the build commandline options, does the following:
---
--- * Parses the build targets.
---
--- * Loads the 'LoadedSnapshot' from the resolver, with extra-deps
---   shadowing any packages that should be built locally.
---
--- * Loads up the 'LocalPackage' info.
---
--- * Builds a 'SourceMap', which contains info for all the packages that
---   will be involved in the build.
-loadSourceMapFull :: HasEnvConfig env
-                  => NeedTargets
-                  -> BuildOptsCLI
-                  -> RIO env
-                       ( Map PackageName Target
-                       , LoadedSnapshot
-                       , [LocalPackage] -- FIXME do we really want this? it's in the SourceMap
-                       , Set PackageName -- non-project targets
-                       , SourceMap
-                       )
-loadSourceMapFull needTargets boptsCli = do
+              -> SMActual DumpedGlobalPackage
+              -> RIO env SourceMap
+loadSourceMap smt boptsCli sma = do
     bconfig <- view buildConfigL
-    (ls, localDeps, targets) <- parseTargets needTargets boptsCli
-    lp <- getLocalPackages
-    locals <- mapM (loadLocalPackage True boptsCli targets) $ Map.toList $ lpProject lp
-    checkFlagsUsed boptsCli locals localDeps (lsPackages ls)
-    checkComponentsBuildable locals
-
-    -- TODO for extra sanity, confirm that the targets we threw away are all TargetAll
-    let nonProjectTargets = Map.keysSet targets `Set.difference` Map.keysSet (lpProject lp)
-
-    -- Combine the local packages, extra-deps, and LoadedSnapshot into
-    -- one unified source map.
-    let goLPI loc n lpi = do
-          let configOpts = getGhcOptions bconfig boptsCli n False False
-          case lpiLocation lpi of
-            -- NOTE: configOpts includes lpiGhcOptions for now, this may get refactored soon
-            PLIndex pir -> return $ PSIndex loc (lpiFlags lpi) configOpts pir
-            PLOther pl -> do
-              root <- view projectRootL
-              lpv <- parseSingleCabalFile root True pl
-              lp' <- loadLocalPackage False boptsCli targets (n, lpv)
-              return $ PSFiles lp' loc
-    sourceMap' <- Map.unions <$> sequence
-      [ return $ Map.fromList $ map (\lp' -> (packageName $ lpPackage lp', PSFiles lp' Local)) locals
-      , sequence $ Map.mapWithKey (goLPI Local) localDeps
-      , sequence $ Map.mapWithKey (goLPI Snap) (lsPackages ls)
-      ]
-    let sourceMap = sourceMap'
-            `Map.difference` Map.fromList (map (, ()) (HashSet.toList wiredInPackages))
-
+    let compiler = smaCompiler sma
+        project = M.map applyOptsFlagsPP $ smaProject sma
+        bopts = configBuild (bcConfig bconfig)
+        applyOptsFlagsPP p@ProjectPackage{ppCommon = c} =
+          p{ppCommon = applyOptsFlags (M.member (cpName c) (smtTargets smt)) True c}
+        deps0 = smtDeps smt <> smaDeps sma
+        deps = M.map applyOptsFlagsDep deps0
+        applyOptsFlagsDep d@DepPackage{dpCommon = c} =
+          d{dpCommon = applyOptsFlags (M.member (cpName c) (smtDeps smt)) False c}
+        applyOptsFlags isTarget isProjectPackage common =
+            let name = cpName common
+                flags = getLocalFlags boptsCli name
+                ghcOptions =
+                  generalGhcOptions bconfig boptsCli isTarget isProjectPackage
+                cabalConfigOpts =
+                  loadCabalConfigOpts bconfig (cpName common) isTarget isProjectPackage
+            in common
+               { cpFlags =
+                     if M.null flags
+                         then cpFlags common
+                         else flags
+               , cpGhcOptions =
+                     ghcOptions ++ cpGhcOptions common
+               , cpCabalConfigOpts =
+                     cabalConfigOpts ++ cpCabalConfigOpts common
+               , cpHaddocks =
+                     if isTarget
+                         then boptsHaddock bopts
+                         else shouldHaddockDeps bopts
+               }
+        packageCliFlags = Map.fromList $
+          mapMaybe maybeProjectFlags $
+          Map.toList (boptsCLIFlags boptsCli)
+        maybeProjectFlags (ACFByName name, fs) = Just (name, fs)
+        maybeProjectFlags _ = Nothing
+        globals = pruneGlobals (smaGlobal sma) (Map.keysSet deps)
+    logDebug "Checking flags"
+    checkFlagsUsedThrowing packageCliFlags FSCommandLine project deps
+    logDebug "SourceMap constructed"
     return
-      ( targets
-      , ls
-      , locals
-      , nonProjectTargets
-      , sourceMap
-      )
+        SourceMap
+        { smTargets = smt
+        , smCompiler = compiler
+        , smProject = project
+        , smDeps = deps
+        , smGlobal = globals
+        }
+
+-- | Get a 'SourceMapHash' for a given 'SourceMap'
+--
+-- Basic rules:
+--
+-- * If someone modifies a GHC installation in any way after Stack
+--   looks at it, they voided the warranty. This includes installing a
+--   brand new build to the same directory, or registering new
+--   packages to the global database.
+--
+-- * We should include everything in the hash that would relate to
+--   immutable packages and identifying the compiler itself. Mutable
+--   packages (both project packages and dependencies) will never make
+--   it into the snapshot database, and can be ignored.
+--
+-- * Target information is only relevant insofar as it effects the
+--   dependency map. The actual current targets for this build are
+--   irrelevant to the cache mechanism, and can be ignored.
+--
+-- * Make sure things like profiling and haddocks are included in the hash
+--
+hashSourceMapData
+    :: (HasBuildConfig env, HasCompiler env)
+    => BuildOptsCLI
+    -> SourceMap
+    -> RIO env SourceMapHash
+hashSourceMapData boptsCli sm = do
+    compilerPath <- getUtf8Builder . fromString . toFilePath <$> getCompilerPath
+    compilerInfo <- getCompilerInfo
+    immDeps <- forM (Map.elems (smDeps sm)) depPackageHashableContent
+    bc <- view buildConfigL
+    let -- extra bytestring specifying GHC options supposed to be applied to
+        -- GHC boot packages so we'll have differrent hashes when bare
+        -- resolver 'ghc-X.Y.Z' is used, no extra-deps and e.g. user wants builds
+        -- with profiling or without
+        bootGhcOpts = map display (generalGhcOptions bc boptsCli False False)
+        hashedContent = toLazyByteString $ compilerPath <> compilerInfo <>
+            getUtf8Builder (mconcat bootGhcOpts) <> mconcat immDeps
+    return $ SourceMapHash (SHA256.hashLazyBytes hashedContent)
+
+depPackageHashableContent :: (HasConfig env) => DepPackage -> RIO env Builder
+depPackageHashableContent DepPackage {..} = do
+    case dpLocation of
+        PLMutable _ -> return ""
+        PLImmutable pli -> do
+            let flagToBs (f, enabled) =
+                    if enabled
+                        then ""
+                        else "-" <> fromString (C.unFlagName f)
+                flags = map flagToBs $ Map.toList (cpFlags dpCommon)
+                ghcOptions = map display (cpGhcOptions dpCommon)
+                cabalConfigOpts = map display (cpCabalConfigOpts dpCommon)
+                haddocks = if cpHaddocks dpCommon then "haddocks" else ""
+                hash = immutableLocSha pli
+            return $ hash <> haddocks <> getUtf8Builder (mconcat flags) <>
+                getUtf8Builder (mconcat ghcOptions) <>
+                getUtf8Builder (mconcat cabalConfigOpts)
 
 -- | All flags for a local package.
 getLocalFlags
-    :: BuildConfig
-    -> BuildOptsCLI
+    :: BuildOptsCLI
     -> PackageName
     -> Map FlagName Bool
-getLocalFlags bconfig boptsCli name = Map.unions
-    [ Map.findWithDefault Map.empty (Just name) cliFlags
-    , Map.findWithDefault Map.empty Nothing cliFlags
-    , Map.findWithDefault Map.empty name (bcFlags bconfig)
+getLocalFlags boptsCli name = Map.unions
+    [ Map.findWithDefault Map.empty (ACFByName name) cliFlags
+    , Map.findWithDefault Map.empty ACFAllProjectPackages cliFlags
     ]
   where
     cliFlags = boptsCLIFlags boptsCli
 
+-- | Get the options to pass to @./Setup.hs configure@
+loadCabalConfigOpts :: BuildConfig -> PackageName -> Bool -> Bool -> [Text]
+loadCabalConfigOpts bconfig name isTarget isLocal = concat
+    [ Map.findWithDefault [] CCKEverything (configCabalConfigOpts config)
+    , if isLocal
+        then Map.findWithDefault [] CCKLocals (configCabalConfigOpts config)
+        else []
+    , if isTarget
+        then Map.findWithDefault [] CCKTargets (configCabalConfigOpts config)
+        else []
+    , Map.findWithDefault [] (CCKPackage name) (configCabalConfigOpts config)
+    ]
+  where
+    config = view configL bconfig
+
 -- | Get the configured options to pass from GHC, based on the build
 -- configuration and commandline.
-getGhcOptions :: BuildConfig -> BuildOptsCLI -> PackageName -> Bool -> Bool -> [Text]
-getGhcOptions bconfig boptsCli name isTarget isLocal = concat
+generalGhcOptions :: BuildConfig -> BuildOptsCLI -> Bool -> Bool -> [Text]
+generalGhcOptions bconfig boptsCli isTarget isLocal = concat
     [ Map.findWithDefault [] AGOEverything (configGhcOptionsByCat config)
     , if isLocal
         then Map.findWithDefault [] AGOLocals (configGhcOptionsByCat config)
@@ -139,7 +214,6 @@ getGhcOptions bconfig boptsCli name isTarget isLocal = concat
     , if isTarget
         then Map.findWithDefault [] AGOTargets (configGhcOptionsByCat config)
         else []
-    , Map.findWithDefault [] name (configGhcOptionsByName config)
     , concat [["-fhpc"] | isLocal && toCoverage (boptsTestOpts bopts)]
     , if boptsLibProfile bopts || boptsExeProfile bopts
          then ["-fprof-auto","-fprof-cafs"]
@@ -172,31 +246,39 @@ splitComponents =
     go a b c (CTest x:xs) = go a (b . (x:)) c xs
     go a b c (CBench x:xs) = go a b (c . (x:)) xs
 
--- | Upgrade the initial local package info to a full-blown @LocalPackage@
+loadCommonPackage ::
+       forall env. (HasBuildConfig env, HasSourceMap env)
+    => CommonPackage
+    -> RIO env Package
+loadCommonPackage common = do
+    config <- getPackageConfig (cpFlags common) (cpGhcOptions common) (cpCabalConfigOpts common)
+    gpkg <- liftIO $ cpGPD common
+    return $ resolvePackage config gpkg
+
+-- | Upgrade the initial project package info to a full-blown @LocalPackage@
 -- based on the selected components
-loadLocalPackage
-    :: forall env. HasEnvConfig env
-    => Bool
-    -- ^ Should this be treated as part of $locals? False for extra-deps.
-    --
-    -- See: https://github.com/commercialhaskell/stack/issues/3574#issuecomment-346512821
-    -> BuildOptsCLI
-    -> Map PackageName Target
-    -> (PackageName, LocalPackageView)
+loadLocalPackage ::
+       forall env. (HasBuildConfig env, HasSourceMap env)
+    => ProjectPackage
     -> RIO env LocalPackage
-loadLocalPackage isLocal boptsCli targets (name, lpv) = do
-    let mtarget = Map.lookup name targets
-    config  <- getPackageConfig boptsCli name (isJust mtarget) isLocal
+loadLocalPackage pp = do
+    sm <- view sourceMapL
+    let common = ppCommon pp
     bopts <- view buildOptsL
-    let (exeCandidates, testCandidates, benchCandidates) =
+    mcurator <- view $ buildConfigL.to bcCurator
+    config <- getPackageConfig (cpFlags common) (cpGhcOptions common) (cpCabalConfigOpts common)
+    gpkg <- ppGPD pp
+    let name = cpName common
+        mtarget = M.lookup name (smtTargets $ smTargets sm)
+        (exeCandidates, testCandidates, benchCandidates) =
             case mtarget of
                 Just (TargetComps comps) -> splitComponents $ Set.toList comps
                 Just (TargetAll _packageType) ->
                     ( packageExes pkg
-                    , if boptsTests bopts
+                    , if boptsTests bopts && maybe True (Set.notMember name . curatorSkipTest) mcurator
                         then Map.keysSet (packageTests pkg)
                         else Set.empty
-                    , if boptsBenchmarks bopts
+                    , if boptsBenchmarks bopts && maybe True (Set.notMember name . curatorSkipBenchmark) mcurator
                         then packageBenchmarks pkg
                         else Set.empty
                     )
@@ -260,7 +342,6 @@ loadLocalPackage isLocal boptsCli targets (name, lpv) = do
         -- This allows us to do an optimization where these are passed
         -- if the deps are present. This can avoid doing later
         -- unnecessary reconfigures.
-        gpkg = lpvGPD lpv
         pkg = resolvePackage config gpkg
         btpkg
             | Set.null tests && Set.null benches = Nothing
@@ -268,21 +349,29 @@ loadLocalPackage isLocal boptsCli targets (name, lpv) = do
         testpkg = resolvePackage testconfig gpkg
         benchpkg = resolvePackage benchconfig gpkg
 
-    (componentFiles,_) <- getPackageFilesForTargets pkg (lpvCabalFP lpv) nonLibComponents
+    componentFiles <- memoizeRefWith $ fst <$> getPackageFilesForTargets pkg (ppCabalFP pp) nonLibComponents
 
-    checkCacheResults <- forM (Map.toList componentFiles) $ \(component, files) -> do
-        mbuildCache <- tryGetBuildCache (lpvRoot lpv) component
+    checkCacheResults <- memoizeRefWith $ do
+      componentFiles' <- runMemoizedWith componentFiles
+      forM (Map.toList componentFiles') $ \(component, files) -> do
+        mbuildCache <- tryGetBuildCache (ppRoot pp) component
         checkCacheResult <- checkBuildCache
             (fromMaybe Map.empty mbuildCache)
             (Set.toList files)
         return (component, checkCacheResult)
 
-    let allDirtyFiles =
-            Set.unions $
-                map (\(_, (dirtyFiles, _)) -> dirtyFiles) checkCacheResults
+    let dirtyFiles = do
+          checkCacheResults' <- checkCacheResults
+          let allDirtyFiles = Set.unions $ map (\(_, (x, _)) -> x) checkCacheResults'
+          pure $
+            if not (Set.null allDirtyFiles)
+                then let tryStripPrefix y =
+                          fromMaybe y (stripPrefix (toFilePath $ ppRoot pp) y)
+                      in Just $ Set.map tryStripPrefix allDirtyFiles
+                else Nothing
         newBuildCaches =
-            M.fromList $
-                map (\(c, (_, cache)) -> (c, cache)) checkCacheResults
+            M.fromList . map (\(c, (_, cache)) -> (c, cache))
+            <$> checkCacheResults
 
     return LocalPackage
         { lpPackage = pkg
@@ -290,16 +379,11 @@ loadLocalPackage isLocal boptsCli targets (name, lpv) = do
         , lpBenchDeps = dvVersionRange <$> packageDeps benchpkg
         , lpTestBench = btpkg
         , lpComponentFiles = componentFiles
+        , lpBuildHaddocks = cpHaddocks (ppCommon pp)
         , lpForceDirty = boptsForceDirty bopts
-        , lpDirtyFiles =
-            if not (Set.null allDirtyFiles)
-                then let tryStripPrefix y =
-                          fromMaybe y (stripPrefix (toFilePath $ lpvRoot lpv) y)
-                      in Just $ Set.map tryStripPrefix allDirtyFiles
-                else Nothing
+        , lpDirtyFiles = dirtyFiles
         , lpNewBuildCaches = newBuildCaches
-        , lpCabalFile = lpvCabalFP lpv
-        , lpDir = lpvRoot lpv
+        , lpCabalFile = ppCabalFP pp
         , lpWanted = isWanted
         , lpComponents = nonLibComponents
         -- TODO: refactor this so that it's easier to be sure that these
@@ -312,54 +396,8 @@ loadLocalPackage isLocal boptsCli targets (name, lpv) = do
             (exes `Set.difference` packageExes pkg)
             (tests `Set.difference` Map.keysSet (packageTests pkg))
             (benches `Set.difference` packageBenchmarks pkg)
-        , lpLocation = lpvLoc lpv
         }
 
--- | Ensure that the flags specified in the stack.yaml file and on the command
--- line are used.
-checkFlagsUsed :: (MonadThrow m, MonadReader env m, HasBuildConfig env)
-               => BuildOptsCLI
-               -> [LocalPackage]
-               -> Map PackageName (LoadedPackageInfo (PackageLocationIndex FilePath)) -- ^ local deps
-               -> Map PackageName snapshot -- ^ snapshot, for error messages
-               -> m ()
-checkFlagsUsed boptsCli lps extraDeps snapshot = do
-    bconfig <- view buildConfigL
-
-        -- Check if flags specified in stack.yaml and the command line are
-        -- used, see https://github.com/commercialhaskell/stack/issues/617
-    let flags = map (, FSCommandLine) [(k, v) | (Just k, v) <- Map.toList $ boptsCLIFlags boptsCli]
-             ++ map (, FSStackYaml) (Map.toList $ bcFlags bconfig)
-
-        localNameMap = Map.fromList $ map (packageName . lpPackage &&& lpPackage) lps
-        checkFlagUsed ((name, userFlags), source) =
-            case Map.lookup name localNameMap of
-                -- Package is not available locally
-                Nothing ->
-                    if Map.member name extraDeps
-                        -- We don't check for flag presence for extra deps
-                        then Nothing
-                        -- Also not in extra-deps, it's an error
-                        else
-                            case Map.lookup name snapshot of
-                                Nothing -> Just $ UFNoPackage source name
-                                Just _ -> Just $ UFSnapshot name
-                -- Package exists locally, let's check if the flags are defined
-                Just pkg ->
-                    let unused = Set.difference (Map.keysSet userFlags) (packageDefinedFlags pkg)
-                     in if Set.null unused
-                            -- All flags are defined, nothing to do
-                            then Nothing
-                            -- Error about the undefined flags
-                            else Just $ UFFlagsNotDefined source pkg unused
-
-        unusedFlags = mapMaybe checkFlagUsed flags
-
-    unless (null unusedFlags)
-        $ throwM
-        $ InvalidFlagSpecification
-        $ Set.fromList unusedFlags
-
 -- | Compare the current filesystem state to the cached information, and
 -- determine (1) if the files are dirty, and (2) the new cache values.
 checkBuildCache :: forall m. (MonadIO m)
@@ -378,7 +416,7 @@ checkBuildCache oldCache files = do
             fileTimes
             oldCache
   where
-    go :: FilePath -> Maybe ModTime -> Maybe FileCacheInfo -> m (Set FilePath, Map FilePath FileCacheInfo)
+    go :: FilePath -> Maybe CTime -> Maybe FileCacheInfo -> m (Set FilePath, Map FilePath FileCacheInfo)
     -- Filter out the cabal_macros file to avoid spurious recompilations
     go fp _ _ | takeFileName fp == "cabal_macros.h" = return (Set.empty, Map.empty)
     -- Common case where it's in the cache and on the filesystem.
@@ -401,7 +439,7 @@ checkBuildCache oldCache files = do
 -- | Returns entries to add to the build cache for any newly found unlisted modules
 addUnlistedToBuildCache
     :: HasEnvConfig env
-    => ModTime
+    => CTime
     -> Package
     -> Path Abs File
     -> Set NamedComponent
@@ -442,25 +480,25 @@ getPackageFilesForTargets pkg cabalFP nonLibComponents = do
     let necessaryComponents = Set.insert CLib $ Set.filter isCInternalLib (M.keysSet components')
         components = necessaryComponents `Set.union` nonLibComponents
         componentsFiles =
-            M.map (\files -> Set.union otherFiles (Set.map dotCabalGetPath files)) $
-                M.filterWithKey (\component _ -> component `Set.member` components) compFiles
+            M.map (\files -> Set.union otherFiles (Set.map dotCabalGetPath $ Set.fromList files)) $
+                M.filterWithKey (\component _ -> component `elem` components) compFiles
     return (componentsFiles, warnings)
 
 -- | Get file modification time, if it exists.
-getModTimeMaybe :: MonadIO m => FilePath -> m (Maybe ModTime)
+getModTimeMaybe :: MonadIO m => FilePath -> m (Maybe CTime)
 getModTimeMaybe fp =
     liftIO
         (catch
              (liftM
-                  (Just . modTime)
-                  (D.getModificationTime fp))
+                  (Just . modificationTime)
+                  (getFileStatus fp))
              (\e ->
                    if isDoesNotExistError e
                        then return Nothing
                        else throwM e))
 
 -- | Create FileCacheInfo for a file.
-calcFci :: MonadIO m => ModTime -> FilePath -> m FileCacheInfo
+calcFci :: MonadIO m => CTime -> FilePath -> m FileCacheInfo
 calcFci modTime' fp = liftIO $
     withSourceFile fp $ \src -> do
         (size, digest) <- runConduit $ src .| getZipSink
@@ -468,39 +506,29 @@ calcFci modTime' fp = liftIO $
                 <$> ZipSink (CL.fold
                     (\x y -> x + fromIntegral (S.length y))
                     0)
-                <*> ZipSink sinkHash)
+                <*> ZipSink SHA256.sinkHash)
         return FileCacheInfo
             { fciModTime = modTime'
-            , fciSize = size
-            , fciHash = Mem.convert (digest :: Digest SHA256)
+            , fciSize = FileSize size
+            , fciHash = digest
             }
 
-checkComponentsBuildable :: MonadThrow m => [LocalPackage] -> m ()
-checkComponentsBuildable lps =
-    unless (null unbuildable) $ throwM $ SomeTargetsNotBuildable unbuildable
-  where
-    unbuildable =
-        [ (packageName (lpPackage lp), c)
-        | lp <- lps
-        , c <- Set.toList (lpUnbuildable lp)
-        ]
-
 -- | Get 'PackageConfig' for package given its name.
-getPackageConfig :: (MonadIO m, MonadReader env m, HasEnvConfig env)
-  => BuildOptsCLI
-  -> PackageName
-  -> Bool
-  -> Bool
-  -> m PackageConfig
-getPackageConfig boptsCli name isTarget isLocal = do
-  bconfig <- view buildConfigL
+getPackageConfig
+  :: (HasBuildConfig env, HasSourceMap env)
+  => Map FlagName Bool
+  -> [Text] -- ^ GHC options
+  -> [Text] -- ^ cabal config opts
+  -> RIO env PackageConfig
+getPackageConfig flags ghcOptions cabalConfigOpts = do
   platform <- view platformL
   compilerVersion <- view actualCompilerVersionL
   return PackageConfig
     { packageConfigEnableTests = False
     , packageConfigEnableBenchmarks = False
-    , packageConfigFlags = getLocalFlags bconfig boptsCli name
-    , packageConfigGhcOptions = getGhcOptions bconfig boptsCli name isTarget isLocal
+    , packageConfigFlags = flags
+    , packageConfigGhcOptions = ghcOptions
+    , packageConfigCabalConfigOpts = cabalConfigOpts
     , packageConfigCompilerVersion = compilerVersion
     , packageConfigPlatform = platform
     }
