@@ -16,8 +16,6 @@ module Stack.BuildPlan
     , checkSnapBuildPlan
     , DepError(..)
     , DepErrors
-    , gpdPackageDeps
-    , gpdPackages
     , removeSrcPkgDefaultFlags
     , selectBestSnapshot
     , showItems
@@ -25,12 +23,10 @@ module Stack.BuildPlan
 
 import           Stack.Prelude hiding (Display (..))
 import qualified Data.Foldable as F
-import qualified Data.HashSet as HashSet
+import qualified Data.Set as Set
 import           Data.List (intercalate)
-import           Data.List.NonEmpty (NonEmpty(..))
 import qualified Data.List.NonEmpty as NonEmpty
 import qualified Data.Map as Map
-import qualified Data.Set as Set
 import qualified Data.Text as T
 import qualified Distribution.Package as C
 import           Distribution.PackageDescription (GenericPackageDescription,
@@ -39,15 +35,13 @@ import           Distribution.PackageDescription (GenericPackageDescription,
 import qualified Distribution.PackageDescription as C
 import           Distribution.System (Platform)
 import           Distribution.Text (display)
+import           Distribution.Types.UnqualComponentName (unUnqualComponentName)
 import qualified Distribution.Version as C
 import qualified RIO
 import           Stack.Constants
 import           Stack.Package
-import           Stack.Snapshot
-import           Stack.Types.BuildPlan
-import           Stack.Types.FlagName
-import           Stack.Types.PackageIdentifier
-import           Stack.Types.PackageName
+import           Stack.SourceMap
+import           Stack.Types.SourceMap
 import           Stack.Types.Version
 import           Stack.Types.Config
 import           Stack.Types.Compiler
@@ -127,7 +121,7 @@ instance Show BuildPlanException where
                 [ packageNameString dep
                 , " (used by "
                 , intercalate ", "
-                    $ map (packageNameString . packageIdentifierName)
+                    $ map (packageNameString . pkgName)
                     $ Set.toList users
                 , ")"
                 ]
@@ -142,22 +136,26 @@ instance Show BuildPlanException where
         ", because no 'compiler' or 'resolver' is specified."
 
 gpdPackages :: [GenericPackageDescription] -> Map PackageName Version
-gpdPackages gpds = Map.fromList $
-            map (fromCabalIdent . C.package . C.packageDescription) gpds
+gpdPackages = Map.fromList . map (toPair . C.package . C.packageDescription)
     where
-        fromCabalIdent (C.PackageIdentifier name version) =
-            (fromCabalPackageName name, fromCabalVersion version)
+        toPair (C.PackageIdentifier name version) = (name, version)
 
 gpdPackageDeps
     :: GenericPackageDescription
-    -> CompilerVersion 'CVActual
+    -> ActualCompiler
     -> Platform
     -> Map FlagName Bool
     -> Map PackageName VersionRange
-gpdPackageDeps gpd cv platform flags =
-    Map.filterWithKey (const . (/= name)) (packageDependencies pkgConfig pkgDesc)
+gpdPackageDeps gpd ac platform flags =
+    Map.filterWithKey (const . not . isLocalLibrary) (packageDependencies pkgConfig pkgDesc)
     where
+        isLocalLibrary name' = name' == name || name' `Set.member` subs
+
         name = gpdPackageName gpd
+        subs = Set.fromList
+             $ map (C.mkPackageName . unUnqualComponentName . fst)
+             $ C.condSubLibraries gpd
+
         -- Since tests and benchmarks are both enabled, doesn't matter
         -- if we choose modified or unmodified
         pkgDesc = pdpModifiedBuildable $ resolvePackageDescription pkgConfig gpd
@@ -166,7 +164,8 @@ gpdPackageDeps gpd cv platform flags =
             , packageConfigEnableBenchmarks = True
             , packageConfigFlags = flags
             , packageConfigGhcOptions = []
-            , packageConfigCompilerVersion = cv
+            , packageConfigCabalConfigOpts = []
+            , packageConfigCompilerVersion = ac
             , packageConfigPlatform = platform
             }
 
@@ -188,10 +187,9 @@ removeSrcPkgDefaultFlags gpds flags =
             let tuples = map getDefault (C.genPackageFlags gpd)
             in Map.singleton (gpdPackageName gpd) (Map.fromList tuples)
 
-        flagName' = fromCabalFlagName . C.flagName
         getDefault f
-            | C.flagDefault f = (flagName' f, True)
-            | otherwise       = (flagName' f, False)
+            | C.flagDefault f = (C.flagName f, True)
+            | otherwise       = (C.flagName f, False)
 
 -- | Find the set of @FlagName@s necessary to get the given
 -- @GenericPackageDescription@ to compile against the given @BuildPlan@. Will
@@ -199,7 +197,7 @@ removeSrcPkgDefaultFlags gpds flags =
 -- Returns the plan which produces least number of dep errors
 selectPackageBuildPlan
     :: Platform
-    -> CompilerVersion 'CVActual
+    -> ActualCompiler
     -> Map PackageName Version
     -> GenericPackageDescription
     -> (Map PackageName (Map FlagName Bool), DepErrors)
@@ -232,13 +230,13 @@ selectPackageBuildPlan platform compiler pool gpd =
             | flagManual f = (fname, flagDefault f) :| []
             | flagDefault f = (fname, True) :| [(fname, False)]
             | otherwise = (fname, False) :| [(fname, True)]
-          where fname = (fromCabalFlagName . flagName) f
+          where fname = flagName f
 
 -- | Check whether with the given set of flags a package's dependency
 -- constraints can be satisfied against a given build plan or pool of packages.
 checkPackageBuildPlan
     :: Platform
-    -> CompilerVersion 'CVActual
+    -> ActualCompiler
     -> Map PackageName Version
     -> Map FlagName Bool
     -> GenericPackageDescription
@@ -292,7 +290,7 @@ combineDepError (DepError a x) (DepError b y) =
 -- will be chosen automatically.
 checkBundleBuildPlan
     :: Platform
-    -> CompilerVersion 'CVActual
+    -> ActualCompiler
     -> Map PackageName Version
     -> Maybe (Map PackageName (Map FlagName Bool))
     -> [GenericPackageDescription]
@@ -316,7 +314,7 @@ data BuildPlanCheck =
       BuildPlanCheckOk      (Map PackageName (Map FlagName Bool))
     | BuildPlanCheckPartial (Map PackageName (Map FlagName Bool)) DepErrors
     | BuildPlanCheckFail    (Map PackageName (Map FlagName Bool)) DepErrors
-                            (CompilerVersion 'CVActual)
+                            ActualCompiler
 
 -- | Compare 'BuildPlanCheck', where GT means a better plan.
 compareBuildPlanCheck :: BuildPlanCheck -> BuildPlanCheck -> Ordering
@@ -342,21 +340,25 @@ instance Show BuildPlanCheck where
 -- the packages.
 checkSnapBuildPlan
     :: (HasConfig env, HasGHCVariant env)
-    => Path Abs Dir -- ^ project root, used for checking out necessary files
-    -> [GenericPackageDescription]
+    => [ResolvedPath Dir]
     -> Maybe (Map PackageName (Map FlagName Bool))
-    -> SnapshotDef
-    -> Maybe (CompilerVersion 'CVActual)
+    -> SnapshotCandidate env
     -> RIO env BuildPlanCheck
-checkSnapBuildPlan root gpds flags snapshotDef mactualCompiler = do
+checkSnapBuildPlan pkgDirs flags snapCandidate = do
     platform <- view platformL
-    rs <- loadSnapshot mactualCompiler root snapshotDef
+    sma <- snapCandidate pkgDirs
+    gpds <- liftIO $ forM (Map.elems $ smaProject sma) (cpGPD . ppCommon)
 
     let
-        compiler = lsCompilerVersion rs
+        compiler = smaCompiler sma
+        globalVersion (GlobalPackageVersion v) = v
+        depVersion dep | PLImmutable loc <- dpLocation dep =
+                           Just $ packageLocationVersion loc
+                       | otherwise =
+                           Nothing
         snapPkgs = Map.union
-          (lpiVersion <$> lsGlobals rs)
-          (lpiVersion <$> lsPackages rs)
+          (Map.mapMaybe depVersion $ smaDeps sma)
+          (Map.map globalVersion $ smaGlobal sma)
         (f, errs) = checkBundleBuildPlan platform compiler snapPkgs flags gpds
         cerrs = compilerErrors compiler errs
 
@@ -369,67 +371,62 @@ checkSnapBuildPlan root gpds flags snapshotDef mactualCompiler = do
     where
         compilerErrors compiler errs
             | whichCompiler compiler == Ghc = ghcErrors errs
-            -- FIXME not sure how to handle ghcjs boot packages
             | otherwise = Map.empty
 
-        isGhcWiredIn p _ = p `HashSet.member` wiredInPackages
+        isGhcWiredIn p _ = p `Set.member` wiredInPackages
         ghcErrors = Map.filterWithKey isGhcWiredIn
 
 -- | Find a snapshot and set of flags that is compatible with and matches as
 -- best as possible with the given 'GenericPackageDescription's.
 selectBestSnapshot
     :: (HasConfig env, HasGHCVariant env)
-    => Path Abs Dir -- ^ project root, used for checking out necessary files
-    -> [GenericPackageDescription]
+    => [ResolvedPath Dir]
     -> NonEmpty SnapName
-    -> RIO env (SnapshotDef, BuildPlanCheck)
-selectBestSnapshot root gpds snaps = do
+    -> RIO env (SnapshotCandidate env, RawSnapshotLocation, BuildPlanCheck)
+selectBestSnapshot pkgDirs snaps = do
     logInfo $ "Selecting the best among "
                <> displayShow (NonEmpty.length snaps)
                <> " snapshots...\n"
-    F.foldr1 go (NonEmpty.map (getResult <=< loadResolver . ResolverStackage) snaps)
+    let resolverStackage (LTS x y) = ltsSnapshotLocation x y
+        resolverStackage (Nightly d) = nightlySnapshotLocation d
+    F.foldr1 go (NonEmpty.map (getResult . resolverStackage) snaps)
     where
         go mold mnew = do
-            old@(_snap, bpc) <- mold
+            old@(_snap, _loc, bpc) <- mold
             case bpc of
                 BuildPlanCheckOk {} -> return old
                 _ -> fmap (betterSnap old) mnew
 
-        getResult snap = do
-            result <- checkSnapBuildPlan root gpds Nothing snap
-              -- We know that we're only dealing with ResolverStackage
-              -- here, where we can rely on the global package hints.
-              -- Therefore, we don't use an actual compiler. For more
-              -- info, see comments on
-              -- Stack.Solver.checkSnapBuildPlanActual.
-              Nothing
-            reportResult result snap
-            return (snap, result)
+        getResult loc = do
+            candidate <- loadProjectSnapshotCandidate loc NoPrintWarnings False
+            result <- checkSnapBuildPlan pkgDirs Nothing candidate
+            reportResult result loc
+            return (candidate, loc, result)
 
-        betterSnap (s1, r1) (s2, r2)
-          | compareBuildPlanCheck r1 r2 /= LT = (s1, r1)
-          | otherwise = (s2, r2)
+        betterSnap (s1, l1, r1) (s2, l2, r2)
+          | compareBuildPlanCheck r1 r2 /= LT = (s1, l1, r1)
+          | otherwise = (s2, l2, r2)
 
-        reportResult BuildPlanCheckOk {} snap = do
-            logInfo $ "* Matches " <> RIO.display (sdResolverName snap)
+        reportResult BuildPlanCheckOk {} loc = do
+            logInfo $ "* Matches " <> RIO.display loc
             logInfo ""
 
-        reportResult r@BuildPlanCheckPartial {} snap = do
-            logWarn $ "* Partially matches " <> RIO.display (sdResolverName snap)
+        reportResult r@BuildPlanCheckPartial {} loc = do
+            logWarn $ "* Partially matches " <> RIO.display loc
             logWarn $ RIO.display $ indent $ T.pack $ show r
 
-        reportResult r@BuildPlanCheckFail {} snap = do
-            logWarn $ "* Rejected " <> RIO.display (sdResolverName snap)
+        reportResult r@BuildPlanCheckFail {} loc = do
+            logWarn $ "* Rejected " <> RIO.display loc
             logWarn $ RIO.display $ indent $ T.pack $ show r
 
         indent t = T.unlines $ fmap ("    " <>) (T.lines t)
 
-showItems :: Show a => [a] -> Text
+showItems :: [String] -> Text
 showItems items = T.concat (map formatItem items)
     where
         formatItem item = T.concat
             [ "    - "
-            , T.pack $ show item
+            , T.pack item
             , "\n"
             ]
 
@@ -449,12 +446,12 @@ showPackageFlags pkg fl =
         formatFlags (f, v) = show f ++ " = " ++ show v
 
 showMapPackages :: Map PackageName a -> Text
-showMapPackages mp = showItems $ Map.keys mp
+showMapPackages mp = showItems $ map packageNameString $ Map.keys mp
 
 showCompilerErrors
     :: Map PackageName (Map FlagName Bool)
     -> DepErrors
-    -> CompilerVersion 'CVActual
+    -> ActualCompiler
     -> Text
 showCompilerErrors flags errs compiler =
     T.concat
