@@ -13,34 +13,31 @@ import qualified Data.Set as Set
 import GHC.IO.Exception
 import Path
 import System.FSNotify
-import System.IO (hPutStrLn, getLine)
+import System.IO (getLine)
+import RIO.PrettyPrint hiding (line)
 
-fileWatch :: Handle
-          -> ((Set (Path Abs File) -> IO ()) -> IO ())
-          -> IO ()
+fileWatch
+  :: (HasLogFunc env, HasTerm env)
+  => ((Set (Path Abs File) -> IO ()) -> RIO env ())
+  -> RIO env ()
 fileWatch = fileWatchConf defaultConfig
 
-fileWatchPoll :: Handle
-              -> ((Set (Path Abs File) -> IO ()) -> IO ())
-              -> IO ()
+fileWatchPoll
+  :: (HasLogFunc env, HasTerm env)
+  => ((Set (Path Abs File) -> IO ()) -> RIO env ())
+  -> RIO env ()
 fileWatchPoll = fileWatchConf $ defaultConfig { confUsePolling = True }
 
 -- | Run an action, watching for file changes
 --
 -- The action provided takes a callback that is used to set the files to be
 -- watched. When any of those files are changed, we rerun the action again.
-fileWatchConf :: WatchConfig
-              -> Handle
-              -> ((Set (Path Abs File) -> IO ()) -> IO ())
-              -> IO ()
-fileWatchConf cfg out inner = withManagerConf cfg $ \manager -> do
-    let putLn = hPutStrLn out
-    outputIsTerminal <- hIsTerminalDeviceOrMinTTY out
-    let withColor color str = putLn $ do
-            if outputIsTerminal
-            then concat [color, str, reset]
-            else str
-
+fileWatchConf
+  :: (HasLogFunc env, HasTerm env)
+  => WatchConfig
+  -> ((Set (Path Abs File) -> IO ()) -> RIO env ())
+  -> RIO env ()
+fileWatchConf cfg inner = withRunInIO $ \run -> withManagerConf cfg $ \manager -> do
     allFiles <- newTVarIO Set.empty
     dirtyVar <- newTVarIO True
     watchVar <- newTVarIO Map.empty
@@ -88,27 +85,26 @@ fileWatchConf cfg out inner = withManagerConf cfg $ \manager -> do
     let watchInput = do
             line <- getLine
             unless (line == "quit") $ do
-                case line of
+                run $ case line of
                     "help" -> do
-                        putLn ""
-                        putLn "help: display this help"
-                        putLn "quit: exit"
-                        putLn "build: force a rebuild"
-                        putLn "watched: display watched files"
+                        logInfo ""
+                        logInfo "help: display this help"
+                        logInfo "quit: exit"
+                        logInfo "build: force a rebuild"
+                        logInfo "watched: display watched files"
                     "build" -> atomically $ writeTVar dirtyVar True
                     "watched" -> do
                         watch <- readTVarIO allFiles
-                        mapM_ putLn (Set.toList watch)
+                        mapM_ (logInfo . fromString) (Set.toList watch)
                     "" -> atomically $ writeTVar dirtyVar True
-                    _ -> putLn $ concat
-                        [ "Unknown command: "
-                        , show line
-                        , ". Try 'help'"
-                        ]
+                    _ -> logInfo $
+                        "Unknown command: " <>
+                        displayShow line <>
+                        ". Try 'help'"
 
                 watchInput
 
-    race_ watchInput $ forever $ do
+    race_ watchInput $ run $ forever $ do
         atomically $ do
             dirty <- readTVar dirtyVar
             check dirty
@@ -123,17 +119,13 @@ fileWatchConf cfg out inner = withManagerConf cfg $ \manager -> do
         -- https://github.com/commercialhaskell/stack/issues/822
         atomically $ writeTVar dirtyVar False
 
-        case eres of
-            Left e -> do
-                let color = case fromException e of
-                        Just ExitSuccess -> green
-                        _ -> red
-                withColor color $ show e
-            _ -> withColor green "Success! Waiting for next file change."
+        prettyInfo $
+          case eres of
+            Left e ->
+                let theStyle = case fromException e of
+                        Just ExitSuccess -> Good
+                        _ -> Error
+                 in style theStyle $ fromString $ show e
+            _ -> style Good "Success! Waiting for next file change."
 
-        putLn "Type help for available commands. Press enter to force a rebuild."
-
-green, red, reset :: String
-green = "\ESC[32m"
-red = "\ESC[31m"
-reset = "\ESC[0m"
+        logInfo "Type help for available commands. Press enter to force a rebuild."
