@@ -1,7 +1,6 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TemplateHaskell #-}
 
 -- | A wrapper around hoogle.
 module Stack.Hoogle
@@ -11,28 +10,37 @@ module Stack.Hoogle
 import           Stack.Prelude
 import qualified Data.ByteString.Lazy.Char8 as BL8
 import           Data.Char (isSpace)
-import           Data.List (find)
-import qualified Data.Set as Set
 import qualified Data.Text as T
+import           Distribution.PackageDescription (packageDescription, package)
+import           Distribution.Types.PackageName (mkPackageName)
+import           Distribution.Version (mkVersion)
+import           Lens.Micro ((?~))
 import           Path (parseAbsFile)
 import           Path.IO hiding (findExecutable)
 import qualified Stack.Build
-import           Stack.Fetch
+import           Stack.Build.Target (NeedTargets(NeedTargets))
 import           Stack.Runners
 import           Stack.Types.Config
-import           Stack.Types.PackageIdentifier
-import           Stack.Types.PackageName
-import           Stack.Types.Version
-import           System.Exit
+import           Stack.Types.SourceMap
+import qualified RIO.Map as Map
 import           RIO.Process
 
+-- | Helper type to duplicate log messages
+data Muted = Muted | NotMuted
+
 -- | Hoogle command.
-hoogleCmd :: ([String],Bool,Bool,Bool) -> GlobalOpts -> IO ()
-hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
+hoogleCmd :: ([String],Bool,Bool,Bool) -> RIO Runner ()
+hoogleCmd (args,setup,rebuild,startServer) =
+  local (over globalOptsL modifyGO) $
+  withConfig YesReexec $
+  withDefaultEnvConfig $ do
     hooglePath <- ensureHoogleInPath
     generateDbIfNeeded hooglePath
     runHoogle hooglePath args'
   where
+    modifyGO :: GlobalOpts -> GlobalOpts
+    modifyGO = globalOptsBuildOptsMonoidL . buildOptsMonoidHaddockL ?~ True
+
     args' :: [String]
     args' = if startServer
                  then ["server", "--local", "--port", "8080"]
@@ -63,82 +71,76 @@ hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
            createDirIfMissing True dir
            runHoogle hooglePath ["generate", "--local"]
     buildHaddocks :: RIO EnvConfig ()
-    buildHaddocks =
-        liftIO
-            (catch
-                 (withBuildConfigAndLock
-                      (set
-                           (globalOptsBuildOptsMonoidL . buildOptsMonoidHaddockL)
-                           (Just True)
-                           go)
-                      (\lk ->
-                            Stack.Build.build
-                                (const (return ()))
-                                lk
-                                defaultBuildOptsCLI))
-                 (\(_ :: ExitCode) ->
-                       return ()))
-    hooglePackageName = $(mkPackageName "hoogle")
-    hoogleMinVersion = $(mkVersion "5.0")
+    buildHaddocks = do
+      config <- view configL
+      runRIO config $ -- a bit weird that we have to drop down like this
+        catch (withDefaultEnvConfig $ Stack.Build.build Nothing)
+              (\(_ :: ExitCode) -> return ())
+    hooglePackageName = mkPackageName "hoogle"
+    hoogleMinVersion = mkVersion [5, 0]
     hoogleMinIdent =
         PackageIdentifier hooglePackageName hoogleMinVersion
-    installHoogle :: RIO EnvConfig ()
-    installHoogle = do
-        hooglePackageIdentifier <-
-            do (_,_,resolved) <-
-                   resolvePackagesAllowMissing
-
-                       -- FIXME this Nothing means "do not follow any
-                       -- specific snapshot", which matches old
-                       -- behavior. However, since introducing the
-                       -- logic to pin a name to a package in a
-                       -- snapshot, we may arguably want to ensure
-                       -- that we're grabbing the version of Hoogle
-                       -- present in the snapshot currently being
-                       -- used.
-                       Nothing
-
-                       mempty
-                       (Set.fromList [hooglePackageName])
-               return
-                   (case find
-                             ((== hooglePackageName) . packageIdentifierName)
-                             (map rpIdent resolved) of
-                        Just ident@(PackageIdentifier _ ver)
-                          | ver >= hoogleMinVersion -> Right ident
-                        _ -> Left hoogleMinIdent)
-        case hooglePackageIdentifier of
-            Left{} -> logInfo $
-              "Minimum " <>
-              display hoogleMinIdent <>
-              " is not in your index. Installing the minimum version."
-            Right ident -> logInfo $
-              "Minimum version is " <>
-              display hoogleMinIdent <>
-              ". Found acceptable " <>
-              display ident <>
-              " in your index, installing it."
+    installHoogle :: RIO EnvConfig (Path Abs File)
+    installHoogle = requiringHoogle Muted $ do
+        Stack.Build.build Nothing
+        mhooglePath' <- findExecutable "hoogle"
+        case mhooglePath' of
+            Right hooglePath -> parseAbsFile hooglePath
+            Left _ -> do
+                logWarn "Couldn't find hoogle in path after installing.  This shouldn't happen, may be a bug."
+                bail
+    requiringHoogle :: Muted -> RIO EnvConfig x -> RIO EnvConfig x
+    requiringHoogle muted f = do
+        hoogleTarget <- do
+          sourceMap <- view $ sourceMapL . to smDeps
+          case Map.lookup hooglePackageName sourceMap of
+            Just hoogleDep ->
+              case dpLocation hoogleDep of
+                PLImmutable pli ->
+                  T.pack . packageIdentifierString <$>
+                      restrictMinHoogleVersion muted (packageLocationIdent pli)
+                plm@(PLMutable _) -> do
+                  T.pack . packageIdentifierString . package . packageDescription
+                      <$> loadCabalFile plm
+            Nothing -> do
+              -- not muted because this should happen only once
+              logWarn "No hoogle version was found, trying to install the latest version"
+              mpir <- getLatestHackageVersion YesRequireHackageIndex hooglePackageName UsePreferredVersions
+              let hoogleIdent = case mpir of
+                      Nothing -> hoogleMinIdent
+                      Just (PackageIdentifierRevision _ ver _) ->
+                          PackageIdentifier hooglePackageName ver
+              T.pack . packageIdentifierString <$>
+                  restrictMinHoogleVersion muted hoogleIdent
         config <- view configL
-        menv <- liftIO $ configProcessContextSettings config envSettings
-        liftIO
-            (catch
-                 (withBuildConfigAndLock
-                      go
-                      (\lk ->
-                            Stack.Build.build
-                                (const (return ()))
-                                lk
-                                defaultBuildOptsCLI
-                                { boptsCLITargets = [ packageIdentifierText
-                                                          (either
-                                                               id
-                                                               id
-                                                               hooglePackageIdentifier)]
-                                }))
-                 (\(e :: ExitCode) ->
-                       case e of
-                           ExitSuccess -> runRIO menv resetExeCache
-                           _ -> throwIO e))
+        let boptsCLI = defaultBuildOptsCLI
+                { boptsCLITargets =  [hoogleTarget]
+                }
+        runRIO config $ withEnvConfig NeedTargets boptsCLI f
+    restrictMinHoogleVersion
+      :: HasLogFunc env
+      => Muted -> PackageIdentifier -> RIO env PackageIdentifier
+    restrictMinHoogleVersion muted ident = do
+      if ident < hoogleMinIdent
+      then do
+          muteableLog LevelWarn muted $
+               "Minimum " <>
+               fromString (packageIdentifierString hoogleMinIdent) <>
+               " is not in your index. Installing the minimum version."
+          pure hoogleMinIdent
+      else do
+          muteableLog LevelInfo muted $
+              "Minimum version is " <>
+              fromString (packageIdentifierString hoogleMinIdent) <>
+              ". Found acceptable " <>
+              fromString (packageIdentifierString ident) <>
+              " in your index, requiring its installation."
+          pure ident
+    muteableLog :: HasLogFunc env => LogLevel -> Muted -> Utf8Builder -> RIO env ()
+    muteableLog logLevel muted msg =
+        case muted of
+            Muted -> pure ()
+            NotMuted -> logGeneric "" logLevel msg
     runHoogle :: Path Abs File -> [String] -> RIO EnvConfig ()
     runHoogle hooglePath hoogleArgs = do
         config <- view configL
@@ -150,7 +152,7 @@ hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
           (hoogleArgs ++ databaseArg)
           runProcess_
     bail :: RIO EnvConfig a
-    bail = liftIO (System.Exit.exitWith (ExitFailure (-1)))
+    bail = exitWith (ExitFailure (-1))
     checkDatabaseExists = do
         path <- hoogleDatabasePath
         liftIO (doesFileExist path)
@@ -158,7 +160,8 @@ hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
     ensureHoogleInPath = do
         config <- view configL
         menv <- liftIO $ configProcessContextSettings config envSettings
-        mhooglePath <- runRIO menv $ findExecutable "hoogle"
+        mhooglePath <- runRIO menv (findExecutable "hoogle") <>
+          requiringHoogle NotMuted (findExecutable "hoogle")
         eres <- case mhooglePath of
             Left _ -> return $ Left "Hoogle isn't installed."
             Right hooglePath -> do
@@ -173,7 +176,7 @@ hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
                         ]
                 return $ case result of
                     Left err -> unexpectedResult $ T.pack (show err)
-                    Right bs -> case parseVersionFromString (takeWhile (not . isSpace) (BL8.unpack bs)) of
+                    Right bs -> case parseVersion (takeWhile (not . isSpace) (BL8.unpack bs)) of
                         Nothing -> unexpectedResult $ T.pack (BL8.unpack bs)
                         Just ver
                             | ver >= hoogleMinVersion -> Right hooglePath
@@ -181,7 +184,7 @@ hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
                                 [ "Installed Hoogle is too old, "
                                 , T.pack hooglePath
                                 , " is version "
-                                , versionText ver
+                                , T.pack $ versionString ver
                                 , " but >= 5.0 is required."
                                 ]
         case eres of
@@ -190,12 +193,6 @@ hoogleCmd (args,setup,rebuild,startServer) go = withBuildConfig go $ do
                 | setup -> do
                     logWarn $ display err <> " Automatically installing (use --no-setup to disable) ..."
                     installHoogle
-                    mhooglePath' <- runRIO menv $ findExecutable "hoogle"
-                    case mhooglePath' of
-                        Right hooglePath -> parseAbsFile hooglePath
-                        Left _ -> do
-                            logWarn "Couldn't find hoogle in path after installing.  This shouldn't happen, may be a bug."
-                            bail
                 | otherwise -> do
                     logWarn $ display err <> " Not installing it due to --no-setup."
                     bail
