@@ -9,33 +9,32 @@ module Stack.Ls
   , listDependenciesCmd
   ) where
 
-import Control.Exception (Exception, throw)
-import Control.Monad.Catch (MonadThrow)
-import Control.Monad.IO.Class (MonadIO, liftIO)
-import Control.Monad.Reader (MonadReader)
-import Control.Monad (when)
+import Control.Exception (throw)
 import Data.Aeson
-import Stack.Prelude
-import Stack.Types.Runner
+import Data.Array.IArray ((//), elems)
+import Stack.Prelude hiding (Snapshot (..))
 import qualified Data.Aeson.Types as A
 import qualified Data.List as L
 import Data.Text hiding (pack, intercalate)
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
-import Data.Typeable (Typeable)
 import qualified Data.Vector as V
-import Network.HTTP.StackClient (httpJSON, getGlobalManager, addRequestHeader, getResponseBody, parseRequest,
-        setRequestManager, hAccept)
+import Network.HTTP.StackClient (httpJSON, addRequestHeader, getResponseBody, parseRequest, hAccept)
 import qualified Options.Applicative as OA
-import Options.Applicative ((<|>))
+import Options.Applicative (idm)
+import Options.Applicative.Builder.Extra (boolFlags)
 import Path
-import Stack.Runners (withBuildConfig, withBuildConfigDot)
-import Stack.Types.Config
+import RIO.PrettyPrint (useColorL)
+import RIO.PrettyPrint.DefaultStyles (defaultStyles)
+import RIO.PrettyPrint.Types (StyleSpec)
+import RIO.PrettyPrint.StylesUpdate (StylesUpdate (..), stylesUpdateL)
 import Stack.Dot
+import Stack.Runners
 import Stack.Options.DotParser (listDepsOptsParser)
-import System.Process.PagerEditor (pageText)
+import Stack.Types.Config
+import System.Console.ANSI.Codes (SGR (Reset), setSGRCode, sgrToCode)
+import System.Process.Pager (pageText)
 import System.Directory (listDirectory)
-import System.IO (stderr, hPutStrLn)
 
 data LsView
     = Local
@@ -50,6 +49,7 @@ data SnapshotType
 data LsCmds
     = LsSnapshot SnapshotOpts
     | LsDependencies ListDepsOpts
+    | LsStyles ListStylesOpts
 
 data SnapshotOpts = SnapshotOpts
     { soptViewType :: LsView
@@ -57,12 +57,18 @@ data SnapshotOpts = SnapshotOpts
     , soptNightlySnapView :: Bool
     } deriving (Eq, Show, Ord)
 
+data ListStylesOpts = ListStylesOpts
+    { coptBasic   :: Bool
+    , coptSGR     :: Bool
+    , coptExample :: Bool
+    } deriving (Eq, Ord, Show)
+
 newtype LsCmdOpts = LsCmdOpts
     { lsView :: LsCmds
     }
 
 lsParser :: OA.Parser LsCmdOpts
-lsParser = LsCmdOpts <$> OA.hsubparser (lsSnapCmd <> lsDepsCmd)
+lsParser = LsCmdOpts <$> OA.hsubparser (lsSnapCmd <> lsDepsCmd <> lsStylesCmd)
 
 lsCmdOptsParser :: OA.Parser LsCmds
 lsCmdOptsParser = LsSnapshot <$> lsViewSnapCmd
@@ -70,6 +76,28 @@ lsCmdOptsParser = LsSnapshot <$> lsViewSnapCmd
 lsDepOptsParser :: OA.Parser LsCmds
 lsDepOptsParser = LsDependencies <$> listDepsOptsParser
 
+lsStylesOptsParser :: OA.Parser LsCmds
+lsStylesOptsParser = LsStyles <$> listStylesOptsParser
+
+listStylesOptsParser :: OA.Parser ListStylesOpts
+listStylesOptsParser = ListStylesOpts
+    <$> boolFlags False
+                  "basic"
+                  "a basic report of the styles used. The default is a fuller \
+                  \one"
+                  idm
+    <*> boolFlags True
+                  "sgr"
+                  "the provision of the equivalent SGR instructions (provided \
+                  \by default). Flag ignored for a basic report"
+                  idm
+    <*> boolFlags True
+                  "example"
+                  "the provision of an example of the applied style (provided \
+                  \by default for colored output). Flag ignored for a basic \
+                  \report"
+                  idm
+
 lsViewSnapCmd :: OA.Parser SnapshotOpts
 lsViewSnapCmd =
     SnapshotOpts <$>
@@ -94,6 +122,19 @@ lsDepsCmd =
         "dependencies"
         (OA.info lsDepOptsParser (OA.progDesc "View the dependencies"))
 
+lsStylesCmd :: OA.Mod OA.CommandFields LsCmds
+lsStylesCmd =
+    OA.command
+        "stack-colors"
+        (OA.info lsStylesOptsParser
+                 (OA.progDesc "View stack's output styles"))
+    <>
+    OA.command
+        "stack-colours"
+        (OA.info lsStylesOptsParser
+                 (OA.progDesc "View stack's output styles (alias for \
+                              \'stack-colors')"))
+
 data Snapshot = Snapshot
     { snapId :: Text
     , snapTitle :: Text
@@ -180,11 +221,9 @@ displayLocalSnapshot term xs = renderData term (localSnaptoText xs)
 localSnaptoText :: [String] -> Text
 localSnaptoText xs = T.intercalate "\n" $ L.map T.pack xs
 
-handleLocal
-    :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
-    => LsCmdOpts -> m ()
+handleLocal :: LsCmdOpts -> RIO Runner ()
 handleLocal lsOpts = do
-    (instRoot :: Path Abs Dir) <- installationRootDeps
+    (instRoot :: Path Abs Dir) <- withConfig YesReexec $ withDefaultEnvConfig installationRootDeps
     isStdoutTerminal <- view terminalL
     let snapRootDir = parent $ parent instRoot
     snapData' <- liftIO $ listDirectory $ toFilePath snapRootDir
@@ -202,17 +241,15 @@ handleLocal lsOpts = do
                     L.filter (L.isPrefixOf "night") snapData
                 _ -> liftIO $ displayLocalSnapshot isStdoutTerminal snapData
         LsDependencies _ -> return ()
+        LsStyles _ -> return ()
 
 handleRemote
-    :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
-    => LsCmdOpts -> m ()
+    :: HasRunner env
+    => LsCmdOpts -> RIO env ()
 handleRemote lsOpts = do
     req <- liftIO $ parseRequest urlInfo
-    mgr <- liftIO getGlobalManager
     isStdoutTerminal <- view terminalL
-    let req' =
-            setRequestManager mgr $
-            addRequestHeader hAccept "application/json" req
+    let req' = addRequestHeader hAccept "application/json" req
     result <- httpJSON req'
     let snapData = getResponseBody result
     case lsView lsOpts of
@@ -228,27 +265,28 @@ handleRemote lsOpts = do
                     filterSnapshotData snapData Nightly
                 _ -> liftIO $ displaySnapshotData isStdoutTerminal snapData
         LsDependencies _ -> return ()
+        LsStyles _ -> return ()
   where
     urlInfo = "https://www.stackage.org/snapshots"
 
-lsCmd :: LsCmdOpts -> GlobalOpts -> IO ()
-lsCmd lsOpts go =
+lsCmd :: LsCmdOpts -> RIO Runner ()
+lsCmd lsOpts =
     case lsView lsOpts of
         LsSnapshot SnapshotOpts {..} ->
             case soptViewType of
-                Local -> withBuildConfig go (handleLocal lsOpts)
-                Remote -> withBuildConfig go (handleRemote lsOpts)
-        LsDependencies depOpts -> listDependenciesCmd False depOpts go
+                Local -> handleLocal lsOpts
+                Remote -> handleRemote lsOpts
+        LsDependencies depOpts -> listDependenciesCmd False depOpts
+        LsStyles stylesOpts -> withConfig NoReexec $ listStylesCmd stylesOpts
 
 -- | List the dependencies
-listDependenciesCmd :: Bool -> ListDepsOpts -> GlobalOpts -> IO ()
-listDependenciesCmd deprecated opts go = do
+listDependenciesCmd :: Bool -> ListDepsOpts -> RIO Runner ()
+listDependenciesCmd deprecated opts = do
     when
         deprecated
-        (hPutStrLn
-             stderr
+        (logWarn
              "DEPRECATED: Use ls dependencies instead. Will be removed in next major version.")
-    withBuildConfigDot (listDepsDotOpts opts) go $ listDependencies opts
+    listDependencies opts
 
 lsViewLocalCmd :: OA.Mod OA.CommandFields LsView
 lsViewLocalCmd =
@@ -261,3 +299,29 @@ lsViewRemoteCmd =
     OA.command
         "remote"
         (OA.info (pure Remote) (OA.progDesc "View remote snapshot"))
+
+-- | List stack's output styles
+listStylesCmd :: ListStylesOpts -> RIO Config ()
+listStylesCmd opts = do
+    lc <- ask
+    -- This is the same test as is used in Stack.Types.Runner.withRunner
+    let useColor = view useColorL lc
+        styles = elems $ defaultStyles // stylesUpdate (view stylesUpdateL lc)
+        isComplex = not (coptBasic opts)
+        showSGR = isComplex && coptSGR opts
+        showExample = isComplex && coptExample opts && useColor
+        styleReports = L.map (styleReport showSGR showExample) styles
+    liftIO $ T.putStrLn $ T.intercalate (if isComplex then "\n" else ":") styleReports
+  where
+    styleReport :: Bool -> Bool -> StyleSpec -> Text
+    styleReport showSGR showExample (k, sgrs) = k <> "=" <> codes
+        <> (if showSGR then sgrsList else mempty)
+        <> (if showExample then example else mempty)
+      where
+        codes = T.intercalate ";" (L.map (fromString . show) $
+                    L.concatMap sgrToCode sgrs)
+        sgrsList = " [" <> T.intercalate ", " (L.map (fromString . show) sgrs)
+                   <> "]"
+        example = " " <> ansi <> "Example" <> reset
+        ansi = fromString $ setSGRCode sgrs
+        reset = fromString $ setSGRCode [Reset]
