@@ -11,9 +11,6 @@ import           Stack.Options.Completion
 import           Stack.Options.PackageParser (readFlag)
 import           Stack.Prelude
 import           Stack.Types.Config
-import           Stack.Types.FlagName
-import           Stack.Types.PackageName
-import           Stack.Types.Version
 
 -- | Parser for CLI-only build arguments
 buildOptsParser :: BuildCommand
@@ -57,6 +54,11 @@ buildOptsParser cmd =
          (long "only-dependencies" <>
           help
               "Only build packages that are dependencies of targets on the command line") <|>
+     flag'
+         BSOnlyLocals
+         (long "only-locals" <>
+          help
+              "Only build packages in the local database, fail if the build plan includes the snapshot database") <|>
      pure BSAll) <*>
     (flag'
          FileWatch
@@ -90,11 +92,9 @@ targetsParser =
              (metavar "TARGET" <>
               completer targetCompleter <>
               help ("If none specified, use all local packages. " <>
-                    "See https://docs.haskellstack.org/en/v" <>
-                    versionString stackMinorVersion <>
-                    "/build_command/#target-syntax for details.")))
+                    "See https://docs.haskellstack.org/en/stable/build_command/#target-syntax for details.")))
 
-flagsParser :: Parser (Map.Map (Maybe PackageName) (Map.Map FlagName Bool))
+flagsParser :: Parser (Map.Map ApplyCLIFlag (Map.Map FlagName Bool))
 flagsParser =
      Map.unionsWith Map.union <$>
      many
