@@ -1,8 +1,9 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 module Stack.Options.GhcVariantParser where
 
-import           Data.Monoid.Extra
 import           Options.Applicative
 import           Options.Applicative.Types         (readerAsk)
+import           Stack.Prelude
 import           Stack.Options.Utils
 import           Stack.Types.Config
 
