@@ -5,27 +5,26 @@ import qualified Data.Map                          as Map
 import           Options.Applicative
 import           Options.Applicative.Types         (readerAsk)
 import           Stack.Prelude
-import           Stack.Types.FlagName
-import           Stack.Types.PackageName
+import           Stack.Types.Config.Build (ApplyCLIFlag (..))
 
 -- | Parser for package:[-]flag
-readFlag :: ReadM (Map (Maybe PackageName) (Map FlagName Bool))
+readFlag :: ReadM (Map ApplyCLIFlag (Map FlagName Bool))
 readFlag = do
     s <- readerAsk
     case break (== ':') s of
         (pn, ':':mflag) -> do
             pn' <-
-                case parsePackageNameFromString pn of
+                case parsePackageName pn of
                     Nothing
-                        | pn == "*" -> return Nothing
+                        | pn == "*" -> return ACFAllProjectPackages
                         | otherwise -> readerError $ "Invalid package name: " ++ pn
-                    Just x -> return $ Just x
+                    Just x -> return $ ACFByName x
             let (b, flagS) =
                     case mflag of
                         '-':x -> (False, x)
                         _ -> (True, mflag)
             flagN <-
-                case parseFlagNameFromString flagS of
+                case parseFlagName flagS of
                     Nothing -> readerError $ "Invalid flag name: " ++ flagS
                     Just x -> return x
             return $ Map.singleton pn' $ Map.singleton flagN b
