@@ -1,20 +1,24 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 
 -- | A ghc-pkg id.
 
 module Stack.Types.GhcPkgId
   (GhcPkgId
+  ,unGhcPkgId
   ,ghcPkgIdParser
   ,parseGhcPkgId
   ,ghcPkgIdString)
   where
 
 import           Stack.Prelude
-import           Data.Aeson.Extended
+import           Pantry.Internal.AesonExtended
 import           Data.Attoparsec.Text
 import qualified Data.Text as T
+import           Database.Persist.Sql (PersistField, PersistFieldSql)
+import           Prelude (Read (..))
 
 -- | A parse fail.
 newtype GhcPkgIdParseFail
@@ -26,14 +30,15 @@ instance Exception GhcPkgIdParseFail
 
 -- | A ghc-pkg package identifier.
 newtype GhcPkgId = GhcPkgId Text
-  deriving (Eq,Ord,Data,Typeable,Generic)
+  deriving (Eq,Ord,Data,Typeable,Generic,PersistField,PersistFieldSql)
 
 instance Hashable GhcPkgId
 instance NFData GhcPkgId
-instance Store GhcPkgId
 
 instance Show GhcPkgId where
   show = show . ghcPkgIdString
+instance Read GhcPkgId where
+  readsPrec i = map (first (GhcPkgId . T.pack)) . readsPrec i
 
 instance FromJSON GhcPkgId where
   parseJSON = withText "GhcPkgId" $ \t ->
@@ -61,3 +66,7 @@ ghcPkgIdParser =
 -- | Get a string representation of GHC package id.
 ghcPkgIdString :: GhcPkgId -> String
 ghcPkgIdString (GhcPkgId x) = T.unpack x
+
+-- | Get a text value of GHC package id
+unGhcPkgId :: GhcPkgId -> Text
+unGhcPkgId (GhcPkgId v) = v
