@@ -8,7 +8,7 @@
 
 module Stack.Types.Nix where
 
-import Data.Aeson.Extended
+import Pantry.Internal.AesonExtended
 import Stack.Prelude
 import Generics.Deriving.Monoid (mappenddefault, memptydefault)
 
@@ -43,7 +43,7 @@ data NixOptsMonoid = NixOptsMonoid
     -- ^ Options to be given to the nix-shell command line
   ,nixMonoidPath :: !(First [Text])
     -- ^ Override parts of NIX_PATH (notably 'nixpkgs')
-  ,nixMonoidAddGCRoots :: !(First Bool)
+  ,nixMonoidAddGCRoots :: !FirstFalse
     -- ^ Should we register gc roots so running nix-collect-garbage doesn't remove nix dependencies
   }
   deriving (Eq, Show, Generic)
@@ -57,7 +57,7 @@ instance FromJSON (WithJSONWarnings NixOptsMonoid) where
               nixMonoidInitFile      <- First <$> o ..:? nixInitFileArgName
               nixMonoidShellOptions  <- First <$> o ..:? nixShellOptsArgName
               nixMonoidPath          <- First <$> o ..:? nixPathArgName
-              nixMonoidAddGCRoots    <- First <$> o ..:? nixAddGCRootsArgName
+              nixMonoidAddGCRoots    <- FirstFalse <$> o ..:? nixAddGCRootsArgName
               return NixOptsMonoid{..})
 
 -- | Left-biased combine Nix options
