@@ -1,5 +1,7 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 module Stack.Build.ExecuteSpec (main, spec) where
 
+import Stack.Prelude
 import Test.Hspec
 
 main :: IO ()
