@@ -10,9 +10,8 @@ import           Data.List ((\\))
 import qualified Data.Map as Map
 import qualified Data.Set as Set
 import           Distribution.License (License (BSD3))
-import           Stack.Prelude
-import           Stack.Types.PackageName
-import           Stack.Types.Version
+import qualified RIO.Text as T
+import           Stack.Prelude hiding (pkgName)
 import           Test.Hspec
 import           Test.Hspec.QuickCheck (prop)
 import           Test.QuickCheck (forAll,choose,Gen)
@@ -20,7 +19,7 @@ import           Test.QuickCheck (forAll,choose,Gen)
 import           Stack.Dot
 
 dummyPayload :: DotPayload
-dummyPayload = DotPayload (parseVersionFromString "0.0.0.0") (Just (Right BSD3))
+dummyPayload = DotPayload (parseVersion "0.0.0.0") (Just (Right BSD3)) Nothing
 
 spec :: Spec
 spec = do
@@ -54,14 +53,14 @@ spec = do
 
     prop "requested packages are pruned" $ do
       let resolvedGraph = runIdentity (resolveDependencies Nothing graph stubLoader)
-          allPackages g = Set.map show (Map.keysSet g `Set.union`  fold (fmap fst g))
+          allPackages g = Map.keysSet g `Set.union` foldMap fst g
       forAll (sublistOf (Set.toList (allPackages resolvedGraph))) $ \toPrune ->
         let pruned = pruneGraph [pkgName "one", pkgName "two"] toPrune resolvedGraph
         in Set.null (allPackages pruned `Set.intersection` Set.fromList toPrune)
 
     prop "pruning removes orhpans" $ do
       let resolvedGraph = runIdentity (resolveDependencies Nothing graph stubLoader)
-          allPackages g = Set.map show (Map.keysSet g `Set.union` fold (fmap fst g))
+          allPackages g = Map.keysSet g `Set.union` foldMap fst g
           orphans g = Map.filterWithKey (\k _ -> not (graphElem k g)) g
       forAll (sublistOf (Set.toList (allPackages resolvedGraph))) $ \toPrune ->
         let pruned = pruneGraph [pkgName "one", pkgName "two"] toPrune resolvedGraph
@@ -74,7 +73,7 @@ sublistOf = filterM (\_ -> choose (False, True))
 
 -- Unsafe internal helper to create a package name
 pkgName :: Text -> PackageName
-pkgName = fromMaybe failure . parsePackageName
+pkgName = fromMaybe failure . parsePackageName . T.unpack
   where
    failure = error "Internal error during package name creation in DotSpec.pkgName"
 
